@@ -4,10 +4,14 @@
 
   esky:  keep frozen apps fresh
 
-Esky is an auto-update framework for frozen Python applications, built on top 
-of bbfreeze.  It provides a simple API through which apps can find, fetch
-and install updates, and a bootstrapping mechanism that keeps the app safe
-in the face of failed or partial updates.
+Esky is an auto-update framework for frozen Python applications.  It provides
+a simple API through which apps can find, fetch and install updates, and a
+bootstrapping mechanism that keeps the app safe in the face of failed or
+partial updates.
+
+Esky is currently capable of freezing apps with bbfreeze or py2exe. Adding
+support for other freezer programs should be straightforward; patches will be
+gratefully accepted.
 
 The main interface is the 'Esky' class, which represents a frozen app.  An Esky
 must be given the path to the top-level directory of the frozen app, and a
@@ -31,20 +35,20 @@ like this:
     prog.exe                 - esky bootstrapping executable
     updates/                 - work area for fetching/unpacking updates
     appname-X.Y.platform/    - specific version of the application
-        prog.exe             - executable(s) as produced by bbfreeze
-        library.zip          - pure-python modules frozen by bbfreeze
+        prog.exe             - executable(s) as produced by freezer module
+        library.zip          - pure-python frozen modules
         pythonXY.dll         - python DLL
         esky-bootstrap.txt   - list of files expected in the bootstrapping env
         ...other deps...
 
-The "appname-X.Y" directory is simply a bbfrozen app directory with some extra
+The "appname-X.Y" directory is simply a frozen app directory with some extra
 bootstrapping information produced by esky.  To freeze your app in such a
 format, there is a "bdist_esky" command that can be used with a standard
-distutils setup.py file.
+distutils setup.py file; see the docstring for esky.bdist_esky for details.
 
 To upgrade to a new version "appname-X.Z", esky performs the following steps:
     * extract it into a temporary directory under "updates"
-    * move all bootstrapping files into "appname-X.Z.platform/esky-bootstrap"
+    * move all bootstrapping files into "appname-X.Z.platm/esky-bootstrap"
     * atomically rename it into the main directory as "appname-X.Z.platform"
     * move contents of "appname-X.Z.platform/esky-bootstrap" into the main dir
     * remove the "appname-X.Z.platform/esky-bootstrap" directory
@@ -62,11 +66,10 @@ call the "cleanup" method on their esky.
 """
 
 __ver_major__ = 0
-__ver_minor__ = 2
-__ver_patch__ = 1
+__ver_minor__ = 3
+__ver_patch__ = 0
 __ver_sub__ = ""
-__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
-                              __ver_patch__,__ver_sub__)
+__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
 
 import sys
@@ -83,10 +86,10 @@ except ImportError:
             
 
 from esky.errors import *
-from esky.bootstrap import split_app_version, parse_version, get_best_version
 from esky.fstransact import FSTransaction
 from esky.finder import SimpleVersionFinder
-from esky.util import is_core_dependency
+from esky.util import is_core_dependency, split_app_version, join_app_version,\
+                      parse_version, get_best_version
 
 
 class Esky(object):
@@ -119,12 +122,11 @@ class Esky(object):
         self._lock_count = 0
         workdir = os.path.join(appdir,"updates")
         if isinstance(version_finder,basestring):
-            self.version_finder = SimpleVersionFinder(appname=self.name,platform=self.platform,workdir=workdir,download_url=version_finder)
-        else:
-            self.version_finder = version_finder
-            self.version_finder.appname = self.name
-            self.version_finder.platform = self.platform
-            self.version_finder.workdir = workdir
+            version_finder = SimpleVersionFinder(download_url=version_finder)
+        version_finder.appname = self.name
+        version_finder.platform = self.platform
+        version_finder.workdir = workdir
+        self.version_finder = version_finder
 
     def reinitialize(self):
         """Reinitialize internal state by poking around in the app directory.
@@ -211,20 +213,32 @@ class Esky(object):
     def cleanup(self):
         """Perform cleanup tasks in the app directory.
 
-        This includes removing older versions of the app and failed update
-        attempts.  Such maintenance is not done automatically since it can
-        take a non-negligible amount of time.
+        This includes removing older versions of the app and completing any
+        failed update attempts.  Such maintenance is not done automatically
+        since it can take a non-negligible amount of time.
         """
+        appdir = self.appdir
         self.lock()
         try:
-            version = get_best_version(self.appdir)
-            manifest = os.path.join(self.appdir,version,"esky-bootstrap.txt")
+            cur_version = get_best_version(appdir)
+            new_version = get_best_version(appdir,include_partial_installs=True)
+            #  If there's a partial install we must complete it, since it
+            #  could have left exes in the bootstrap env and we don't want
+            #  to accidentally delete their dependencies.
+            if cur_version != new_version:
+                (_,v,_) = split_app_version(new_version)
+                self.install_update(v)
+                cur_version = new_version
+            #  Now we can safely remove anything that's not part of the
+            #  current version's bootstrap env.
+            manifest = os.path.join(appdir,cur_version,"esky-bootstrap.txt")
             manifest = [ln.strip() for ln in open(manifest,"rt")]
-            for nm in os.listdir(self.appdir):
-                fullnm = os.path.join(self.appdir,nm)
+            for nm in os.listdir(appdir):
+                fullnm = os.path.join(appdir,nm)
                 if os.path.isdir(fullnm):
-                    if nm not in ("updates","locked",version):
-                        shutil.rmtree(fullnm)
+                    if nm not in ("updates","locked",cur_version):
+                        if nm not in manifest:
+                            shutil.rmtree(fullnm)
                 else:
                     if nm not in manifest:
                         os.unlink(fullnm)
@@ -258,7 +272,8 @@ class Esky(object):
         before proceeding.
         """
         #  Extract update then rename into position in main app directory
-        target = os.path.join(self.appdir,"%s-%s.%s"%(self.name,version,self.platform,))
+        target = join_app_version(self.name,version,self.platform)
+        target = os.path.join(self.appdir,target)
         if not os.path.exists(target):
             if not self.version_finder.has_version(version):
                 self.version_finder.fetch_version(version)
@@ -295,7 +310,7 @@ class Esky(object):
                 #  Remove/disable the old version.
                 #  On win32 we can't remove in-use files, so just clobber
                 #  library.zip and leave to rest to a cleanup() call.
-                oldv = "%s-%s.%s" % (self.name,self.version,self.platform)
+                oldv = join_app_version(self.name,self.version,self.platform)
                 oldv = os.path.join(self.appdir,oldv)
                 if sys.platform == "win32":
                     trn.remove(os.path.join(oldv,"library.zip"))
