@@ -5,8 +5,9 @@
   esky.bootstrap:  minimal bootstrapping code for esky
 
 This module provides the minimal code necessary to bootstrap a frozen
-application packaged using esky.  It checks the runtime directory to find
-the most appropriate version of the app and then execvs to the frozen exe.
+application packaged using esky.  It checks the base runtime directory to
+find the most appropriate version of the app and then execvs to the frozen
+executable.
 
 This module must use no modules other than builtins, since the stdlib is
 not available in the bootstrapping environment.  It must also be capable
@@ -14,12 +15,15 @@ of bootstrapping into apps made with older versions of esky, since a partial
 update could result in the boostrapper from a new version being forced
 to load an old version.
 
-The code from this module is included in the bootstrapping environment under
-the module name "bootstrap". 
+The code from this module is always executed in the bootstrapping environment
+before any custom bootstrapping code.  It provides the following functions for
+use during the bootstrap process:
+
+  Chainloading:         execv, chainload
+  Filesystem:           listdir, exists, basename, dirname, pathjoin
+  Version handling:     split_app_version, join_app_version, parse_version,
+                        get_best_version
 
-If I can be bothered doing all this in C, I plan to eventually replace this
-module with a custom program loader. For now, this lets us get up and running
-with minimal effort.
 
 """
 
@@ -34,6 +38,9 @@ if "posix" in sys.builtin_module_names:
 elif "nt" in  sys.builtin_module_names:
     from nt import listdir, stat, unlink, rename, spawnv, P_WAIT
     SEP = "\\"
+    def execv(filename,args):
+        res = spawnv(P_WAIT,filename,args)
+        raise SystemExit(res)
 else:
     raise RuntimeError("unsupported platform: " + sys.platform)
 
@@ -64,37 +71,30 @@ def exists(path):
 
 
 def bootstrap():
-    """Bootstrap an esky frozen app into newest available version."""
-    #  bbfreeze always sets sys.path to [appdir/library.zip,appdir]
-    appdir = sys.path[1]
+    """Bootstrap an esky frozen app into the newest available version.
+
+    This function searches the application directory to find the highest-
+    numbered version of the application that is fully installed, then
+    chain-loads that version of the application.
+    """
+    appdir = dirname(sys.executable)
     best_version = get_best_version(appdir)
     if best_version is None:
         raise RuntimeError("no usable frozen versions were found")
-    target_exe = pathjoin(appdir,best_version,basename(sys.executable))
-    if sys.platform != "win32":
-        #  Non-Windows platforms have a functioning, low-overhead execv,
-        #  so we just replace ourself with the target executable.
-        execv(target_exe,[target_exe] + sys.argv[1:])
-    else:
-        #  On Windows, execv is flaky and expensive.  If we're the same python
-        #  version as the target exe, munge sys.path to bootstrap it directly.
-        #  If not, we'll have to spawn the target exe and wait for it to return.
-        pydll = "python%s%s.dll" % sys.version_info[:2]
-        if exists(pathjoin(appdir,best_version,pydll)):
-            sys.executable = target_exe
-            sys.argv[0] = target_exe
-            del sys.path[:]
-            sys.path.append(pathjoin(appdir,best_version,"library.zip"))
-            sys.path.append(pathjoin(appdir,best_version))
-            import zipimport
-            importer = zipimport.zipimporter(sys.path[0])
-            exec importer.get_code("__main__") in {}
-        else:
-            res = spawnv(P_WAIT,target_exe,[target_exe] + sys.argv[1:])
-            raise SystemExit(res)
+    return chainload(pathjoin(appdir,best_version))
 
 
-def get_best_version(appdir):
+def chainload(target_dir):
+    """Load and execute the selected version of an application.
+
+    This function replaces the currently-running executable with the equivalent
+    executable from the given target directory.
+    """
+    target_exe = pathjoin(target_dir,basename(sys.executable))
+    execv(target_exe,[target_exe] + sys.argv[1:])
+
+
+def get_best_version(appdir,include_partial_installs=False):
     """Get the best usable version from within the given appdir.
 
     In the common case there is only a single version directory, but failed
@@ -102,20 +102,22 @@ def get_best_version(appdir):
     finds the highest-numbered version that is completely installed.
     """
     #  Find all potential version directories, sorted by version number.
-    #  To be a version directory, it must contain a "library.zip".
+    #  To be a version directory, it must contain an "esky-bootstrap.txt" file.
     candidates = []
     for nm in listdir(appdir):
         (_,ver,_) = split_app_version(nm)
-        if ver and exists(pathjoin(appdir,nm,"library.zip")):
+        if ver and exists(pathjoin(appdir,nm,"esky-bootstrap.txt")):
             ver = parse_version(ver)
             candidates.append((ver,nm))
     candidates = [c[1] for c in sorted(candidates,reverse=True)]
-    #  In the (hopefully) common case of no failed upgrade, we don't need
+    #  In the (hopefully) common case of no failed updates, we don't need
     #  to poke around in the filesystem so we just return asap.
     if not candidates:
         return None
     if len(candidates) == 1:
         return candidates[0]
+    if include_partial_installs:
+        return candidates[0]
     #  If there are several candidate versions, we need to find the best
     #  one whose 'esky-bootstrap' dir has been completely removed.
     while candidates:
@@ -126,7 +128,7 @@ def get_best_version(appdir):
 
 
 def split_app_version(s):
-    """Split a app version string to name, version and platform components.
+    """Split an app version string to name, version and platform components.
 
     For example, app-name-0.1.2.win32 => ("app-name","0.1.2","win32")
     """
@@ -142,6 +144,14 @@ def split_app_version(s):
     version = ".".join(bits[:-1])
     platform = bits[-1]
     return (appname,version,platform)
+
+
+def join_app_version(appname,version,platform):
+    """Join an app name, version and platform into a version directory name.
+
+    For example, ("app-name","0.1.2","win32") => appname-0.1.2.win32
+    """
+    return "%s-%s.%s" % (appname,version,platform,)
     
 
 def parse_version(s):
